/**
 * @author dochmann
 */

import mx.utils.Delegate;
import mx.events.EventDispatcher;	
import de.davidochmann.utilities.dogesture.move.Move;	 
import de.davidochmann.utilities.dogesture.logic.Logic;	
import de.davidochmann.utilities.dogesture.stack.Stack;	
import de.davidochmann.utilities.dogesture.register.Register;
import de.davidochmann.utilities.dogesture.events.GestureEvent;	

class de.davidochmann.utilities.dogesture.DoGesture
{	
	private var mSurface:MovieClip;
	private var mTimeout:Number;
	private var mMoveArray:Array;
	private var mRegister:Register;

	private var mStack:Stack;
	private var mLogic:Logic;
	

	public function DoGesture(pSurface:MovieClip,
							  pTimeout:Number,
							  pMoveArray:Array,
							  pStart:Boolean)
	{
		super();
		EventDispatcher.initialize(this);
		
		setSurface(pSurface);
		setTimeout(pTimeout);
		setMoveArray(pMoveArray);
		
		initStack();
		initLogic();
		initEventHandler();
		initMoveArray();
		initRegister();
		
		if(pStart)
			start();
	}
	
	
	
	public function start():Void
	{
		mRegister.setSurface(mSurface);
		mRegister.start();
		
		mLogic.setSurface(mSurface);
		mLogic.setStack(mStack);
		mLogic.setTimeout(mTimeout);
		mLogic.start();
	}
	
	public function stop():Void
	{
		mLogic.stop();
	}
	
	public function add(pPositionX:Number, pPositionY:Number):Void
	{			
		var move:Move = new Move(pPositionX, pPositionY);
		mStack.add(move);
	}

	
	public function getSurface():MovieClip
	{
		return mSurface;
	}

	public function getTimeout():Number
	{
		return mTimeout;
	}

	public function getMoveArray():Array
	{
		return mMoveArray;
	}

	
	public function setSurface(pSurface:MovieClip):Void
	{
		mSurface = pSurface;	
	}

	public function setTimeout(pTimeout:Number):Void
	{					
		mTimeout = pTimeout;
	}

	public function setMoveArray(pMoveArray:Array):Void
	{
		mMoveArray = pMoveArray;
	}

	
	private function initStack():Void
	{
		mStack = new Stack();
	}
	
	private function initLogic():Void
	{
		mLogic = new Logic();	
	}
	
	private function initEventHandler():Void
	{
		mLogic.addEventListener(GestureEvent.MOVE_COMPLETE, Delegate.create(this, logicMoveCompleteHandler));
		mLogic.addEventListener(GestureEvent.GESTURE_COMPLETE, Delegate.create(this, logicMoveCompleteHandler));
	}
	
	private function initMoveArray():Void
	{
		if(mMoveArray == null)
			return;
			
		var obj:Object;
		var move:Move;
			
		for(var i:Number = 0; i < mMoveArray.length; ++i)
		{
			obj  = Object(mMoveArray[i]);
			move = new Move(obj._x, obj._y);
			
			mStack.add(move);
		}			
	}
	
	private function initRegister():Void
	{
		mRegister = Register.getInstance();
	}

	private function logicMoveCompleteHandler(e:GestureEvent):Void
	{
		dispatchEvent(e);
	}
	
	
	public function addEventListener(type:String, listener:Object):Void{};
	
	public function removeEventListener(type:String, listener:Object):Void{};
	
	private function dispatchEvent(event:Object):Void{};
}

